@@ -157,7 +157,7 @@ on_url_home_activate (GtkAction * action, gpointer user_data)
 void
 on_url_bugs_activate (GtkAction * action, gpointer user_data)
 {
-	anjuta_res_url_show("http://bugzilla.gnome.org/simple-bug-guide.cgi");
+	anjuta_res_url_show("https://bugzilla.gnome.org/enter_bug.cgi?product=anjuta");
 }
 
 void
